use Test::More;
BEGIN {
	use Hades;
	Hades->run({
		eval => 'Kosmos::Errors base String::Locale::Object { 
			_build_locales $values :t(HashRef) :d({}) { 
				return {
					stranger => {
						en_GB => "Hello stranger",
						en_US => "Howdy stranger"
					},
					%{$values}
				};
			}
		}',
		lib => 't/lib'
	});
	use lib 't/lib';
}

use String::Locale {
	stash => 'Kosmos::Errors'
};

eval { die locales->stranger };
like($@, qr/(Hello|Howdy) stranger/);

done_testing;
