use Test::More;
use String::Locale {
	stranger => {
		en_GB => 'Hello stranger',
		en_US => 'Howdy stranger',
	}
};
eval { die locales->stranger };
like($@, qr/(Hello|Howdy) stranger/);

done_testing(1);
